#!/usr/bin/perl
#
# $Id: consolidate.pl,v 1.4 1998/05/07 18:45:21 wralph Exp $
#
#  Script to uncompress the hourly tcpdump data files, put them all
#  back into one giant file and then create separate files for the
#  major protocols.
#  
#  Written by Bill Ralph 3/17/98
#
# Set up some variables.
#
use Getopt::Long;
use POSIX qw(strftime);
#
$CID_PATH = "/usr/local/logger";
$ENV{PATH} = "/bin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin:$CID_PATH";
#
sub usage {
        print "Usage: consolidate.pl -debug -d Date -l Site.\n";
	exit 2;
}
#
#	Parse the command line parameters.
#
#
&GetOptions("debug", \$debug, "d:s", \$snifdate, "l=s", \$Site);
##
#
#  Check parameter validity.
#
if ("$Site" eq "")
{
	usage();
}

##
#
#  Rid ourselves of standard output and error to prevent tons of mail messages
#  from cron.
#
@lvls = split(/\//,$0);				# Split path
($call, $suf) = split(/\./, $lvls[$#lvls]);	# Split call.suffix
$LOG_FILE = $debug ? "/tmp/$call.log" : "/dev/null";
open(STDOUT, ">>$LOG_FILE");
select STDOUT; $| = 1;
open(STDERR, ">&STDOUT");
select STDERR; $| = 1;              # make unbuffered

#
# Once the Site is identified from the command line, 
# load the needed external Parameters.
# 
unshift(@INC, "$CID_PATH/sites");
require "${Site}.ph";

#

   print "Command line Date = $snifdate \n";
   print "Command line Site = $Site \n";

#
# Writer a marker to the log file.
#

$marker=strftime("%c", localtime(time));
print STDOUT $marker;
foreach $param ($0, @ARGV) {
   printf " %s", $param;
}
print STDOUT "\n";

#
# Set the $subdir variable from the command line input and/or site params.
#
if ($snifdate ne "") {
   $year=substr($snifdate,0,2);
   $mon=substr($snifdate,2,2) - 1;
   $mday=substr($snifdate,4,2);
   $subdir=strftime("%h%d", 0, 0, 0, $mday, $mon, $year, 0, 0, 0);
} else {
#
   $subdir=strftime("%h%d", localtime($CONSOLIDATE_TIME));
}

   print STDOUT "subdir = $subdir\n";

#
# Ready to start the work now, find the hourly data files.
#

   print STDOUT ("${ANALYZER_DIR}/${subdir}\n");
#
# unzip the hourly data files.
#
#
#
chdir("${ANALYZER_DIR}/${subdir}");
@files=<tcp.*.gz>;
if ($files[0] eq "") {
   print STDOUT ("No zipped hourly files found, moving on.\n");
} else {
   foreach $file (@files) {
      @file_stat = stat($file);
#
# If the size of the gzipped file is less that 30 bytes, don't bother.
#
      if ($file_stat[7] < 30) {
         unlink($file);
         next;
      }
      print STDOUT ("Unzipping $file\n");
      
      system("gunzip -f $file") == 0 or unlink($file);
      ($new_fn,$ext) = split(/\./,$file,2);
      @file_stat = stat($new_fn);
      if ($file_stat[7] < 30) {
         unlink($new_fn);
         next;
      }
   }
}
#
#  Build an entire day file from the hourly pieces.
#
@files=<tcp.*>;
if ($files[0] ne "") {
   $day_whole = "/spare/tmp/day_whole_$$";
   if (! -e "$day_whole") {
      print STDOUT ("tcpslice -w $day_whole tcp.*\n");
      system("tcpslice -w $day_whole tcp.*") == 0 or die "tcpslice failed: $?";
   }
#
#  Remove the hourly pieces
#
   foreach $file (@files) {
      print STDOUT ("Removing $file\n");
      if (unlink($file) == 0) {
            print STDOUT "Failed.\n"; 
      }
   }
} else {
   print STDOUT ("No unzipped hourly files found: Exiting.\n");
   exit 0;
}
#
#  Separate the whole day into tcp, udp, icmp, and everything_else pieces.
#  Combine the tcpdumping with the gzipping at the suggestion of Andy Kutner
#  (akutner@lbl.gov).
#
if (! -e "day_tcp.gz") {
   $tcpdump_cmd="tcpdump -r $day_whole -w - -F $GENERIC_FILTERS/tcp-connections";
   $gzip_cmd="gzip -f -c > day_tcp.gz";
   print STDOUT ("$tcpdump_cmd | $gzip_cmd\n");
   system("$tcpdump_cmd | $gzip_cmd") == 0 or 
        die "tcpdump/gzip failed on tcp-connections: $?";
}
if (! -e "day_udp.gz") {
   $tcpdump_cmd="tcpdump -r $day_whole -w - -F $GENERIC_FILTERS/udp";
   $gzip_cmd="gzip -f -c > day_udp.gz";
   print STDOUT ("$tcpdump_cmd | $gzip_cmd\n");
   system("$tcpdump_cmd | $gzip_cmd") == 0 or 
        die "tcpdump/gzip failed on udp: $?";
}
if (! -e "day_icmp.gz") {
   $tcpdump_cmd="tcpdump -r $day_whole -w - -F $GENERIC_FILTERS/icmp";
   $gzip_cmd="gzip -f -c > day_icmp.gz";
   print STDOUT ("$tcpdump_cmd | $gzip_cmd\n");
   system("$tcpdump_cmd | $gzip_cmd") == 0 or 
        die "tcpdump/gzip failed on icmp: $?";
}
if (! -e "day_else.gz") {
   $tcpdump_cmd="tcpdump -r $day_whole -w - -F $GENERIC_FILTERS/everything-else";
   $gzip_cmd="gzip -f -c > day_else.gz";
   print STDOUT ("$tcpdump_cmd | $gzip_cmd\n");
   system("$tcpdump_cmd | $gzip_cmd") == 0 or 
        die "tcpdump/gzip failed on everything-else: $?";
}
#
#  Now remove the whole day file.
#
unlink($day_whole) or die "unlink failed on $day_whole.\n";
$marker=strftime("%c", localtime(time));
print STDOUT "consolidate.pl completed at $marker.\n";
